/*

-Procedure turnpar ( Determine rate prameters of a spacecraft turn. )

   Jeff Bytof, last update 14-May-2001

*/

#include "SpiceUsr.h"
#include "SpiceZmc.h"
#include "SpiceZfc.h"

void turnpar ( SpiceInt         opt, 
               SpiceDouble      r1        [3], 
               SpiceDouble      r2        [3],
               SpiceDouble      t,
               SpiceDouble    * alpha,
               SpiceDouble    * tb,
               SpiceDouble    * tc,
               SpiceDouble      omega     [3],
               SpiceDouble      angacc    [3],
               SpiceBoolean   * found,
               SpiceInt       * ier                   
             )

{


/*

-Brief_I/O

    Variable    I/O      Description
    --------    ---      -----------
    opt          I       Profile calculation option
                         1 = input r1, r2, t and acc to find TB.
                         2 = input r1, r2, t and tb to find acc.
    r1           I       Initial vector, spacecraft frame.
    r2           I       Final vector of turn, spacecraft frame.
    t            I       Total duration of turn, in seconds.
 
    alpha       I/O      Absolute angular acceleration of spinup, in rad/sec^2.
    tb          I/O      Duration of spinup phase, in seconds.
 
    tc           O       Duration of unaccelerated coast, in seconds.
    omega        O       Overall angular velocity components, in rad/sec.
    angacc       O       Boost angular acceleration components.  Negate
                           these for spindown.
    found        O       Logical flag, SPICETRUE if value for tb or acc is 
                           found.
    ier          O       If found = SPICEFALSE returns an error code.

*/
   
/*
   Local variables.
*/

   SpiceDouble   a;
   SpiceDouble   angvec     [3]; 
   SpiceDouble   b;
   SpiceDouble   c;
   SpiceDouble   disc;
   SpiceDouble   root1      [2];
   SpiceDouble   root2      [2];
   SpiceDouble   theta;
 
   SpiceInt      i;


/*
   Set output flags.
*/

   *found = SPICEFALSE;
   *ier   = 0;


/*
   Check for bad inputs. 
*/
 
   if ( vzero_c( r1 ) || vzero_c( r2 ) || ( t <= 0.0 ) ) {
      *ier = 1;
      return;
   }

/*
   Compute complete turn angle ( 0.0 < theta <= 180.0 ).
*/

   theta = vsep_c ( r1, r2 );

   if ( theta == 0.0 ) {
      *ier = 2;
      return;
   }

/***********************************************************
     Boost duration given, find angular acceleration.
*/

   if ( opt == 1 ) {
   
      *alpha = theta / ( *tb * ( t - *tb ) );

/*
   Verify that boost duration is within bounds.
*/

      if ( ( *tb*2.0 > t ) || ( *tb < 0.0 ) ) {
         *ier = 3;
         return;
      }

/***********************************************************
    Angular acceleration given, find boost duration.
*/

    } else if ( opt == 2 ) { 

/*
    Setup for quadration formula.
*/

       a = -(*alpha);
       b =  (*alpha) * t; 
       c = -theta;

/*
   Check for sign of discriminant, and check for imaginary solutions. 
*/

       disc = b*b - 4.0*a*c;

       if ( disc < 0.0 ) {
          *ier = 4;
          return;
       }

       rquad_c ( a, b, c, root1, root2 );

/*
   Use the second root.
*/

      *tb = root1[0];
      *tc = t - *tb*2.0;

   } else { 

/*
   Illegal option value.
*/

      *ier = 5;
      return;

   }

/*
   Determine coast duration.
*/

   *tc = t - (*tb)*2.0;

/*
   Obtain angular velocity unit vector.  
*/

   ucrss_c ( r1, r2, angvec );

/*
   Obtain overall angular velocity vector.
*/ 

   vscl_c ( theta/t, angvec, omega );

/*
   Obtain components of spinup angular acceleration.
*/

   for ( i=0; i<3; i++ ) {
      angacc[i] = (*alpha) * angvec[i];
   }

   *found = SPICETRUE;

   return;

}
